﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking.Interfaces;

namespace HeuristicLab.Encodings.PackingEncoding.GroupingVector {
  [Item("Grouping Vector Discrete Crossover", "An operator which crosses two GroupingVector representations.")]
  [StorableClass]
  public class GroupingVectorDiscreteCrossover : GroupingVectorCrossover, IGroupingVectorOperator {

    [StorableConstructor]
    protected GroupingVectorDiscreteCrossover (bool deserializing) : base(deserializing) { }
    protected GroupingVectorDiscreteCrossover(GroupingVectorDiscreteCrossover original, Cloner cloner) : base(original, cloner) { }
    public GroupingVectorDiscreteCrossover()
      : base() {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new GroupingVectorDiscreteCrossover(this, cloner);
    }

    public override GroupingVectorEncoding Cross(IRandom random, GroupingVectorEncoding parent1, GroupingVectorEncoding parent2) {
      GroupingVectorEncoding result = new GroupingVectorEncoding();
      result.GroupingVector = DiscreteCrossover.Apply(
        random, new ItemArray<IntegerVector> (new IntegerVector[]{parent1.GroupingVector, parent2.GroupingVector}));
      return result;
    }
  }
}
